
import br.shmem.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

public class StringBufferClient extends JFrame {

	StringBufferReference buffer;
	JTextField field;
	JLabel contents;
	
	public StringBufferClient(String globalName, StringBufferReference buffer) {
		this.buffer = buffer;
		field = new JTextField();
		contents = new JLabel();
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(makeCenterPanel(globalName), BorderLayout.CENTER);
		getContentPane().add(makeButtonPanel(), BorderLayout.EAST);
		read();
	}
	
	protected JComponent makeCenterPanel(String title) {
		JPanel panel = new JPanel(new GridLayout(0, 1));
		
		Box box = Box.createHorizontalBox();
		box.add(Box.createHorizontalGlue());
		box.add(new JLabel("Input:"));
		box.add(Box.createHorizontalStrut(5));
		box.add(field);
		box.add(Box.createHorizontalGlue());
		
		panel.add(box);
		
		box = Box.createHorizontalBox();
		box.add(Box.createHorizontalGlue());
		box.add(new JLabel("Buffer contents:"));
		box.add(Box.createHorizontalStrut(5));
		box.add(contents);
		box.add(Box.createHorizontalGlue());
		
		panel.add(box);
		
		panel.setBorder(BorderFactory.createTitledBorder(title));
		return panel;
	}
	
	public void clear() {
		try {
			buffer.delete(0, buffer.length());
			read();
		} catch (ShmemException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Clear Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	public void append() {
		try {
			buffer.append(field.getText());
			read();
		} catch (ShmemException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Append Error", JOptionPane.ERROR_MESSAGE);
		}

	}
	
	public void read() {
		try {
			contents.setText(buffer.toStringProxy());
		} catch (ShmemException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Read Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	protected JComponent makeButtonPanel() {
		JPanel p = new JPanel(new BorderLayout());
		JButton append = new JButton("Append");
		
		append.addActionListener(new  ActionListener() {
            public void actionPerformed(ActionEvent e) {
                append();
            };
        });
				
		JButton clear = new JButton("Clear");
		
		clear.addActionListener(new  ActionListener() {
            public void actionPerformed(ActionEvent e) {
                clear();
            };
        });
				
		JButton read = new JButton("Read again");
		
		read.addActionListener(new  ActionListener() {
            public void actionPerformed(ActionEvent e) {
                read();
            };
        });
		
		Box b = Box.createVerticalBox();
		b.add(Box.createVerticalGlue());
		b.add(append);
		b.add(Box.createVerticalStrut(10));
		b.add(clear);
		b.add(Box.createVerticalStrut(10));
		b.add(read);
		
		p.add(b, BorderLayout.CENTER);
		p.setBorder(BorderFactory.createEtchedBorder());
		
		return p;
	}
	
	public static final void main(String args[]) 
		throws Exception
	{
		Properties config = new Properties();
		config.load(new java.io.FileInputStream("client.ini"));
		String url = config.getProperty("url");
		int port = Integer.parseInt(config.getProperty("port"));
		SharedMemory shmem = new SharedMemory(url, port);
		String servername = config.getProperty("server_url");
		int serverport = Integer.parseInt(config.getProperty("server_port"));
		shmem.connect(servername, serverport);
		if (args.length > 0) {
			StringBufferReference buffer = (StringBufferReference) shmem.get(args[0]);
			StringBufferClient client = new StringBufferClient(args[0], buffer);
			client.pack();
			client.setVisible(true);
		}
		else {
			System.out.println("please supply a global name to get the reference from");
			System.exit(-1);
		}
	}
	
}
