package br.shmem.compiler;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import org.jdom.*;
import org.jdom.input.*;
import java.io.*;
import java.util.*;

public class Configurator extends JFrame {
    
    ReferenceDescriptor descriptor;
    ReferenceModel model;
    JTable table;
    JButton generate;
    JButton close;
    JTextField name;
		JTextField fileName;
    
    public Configurator(ReferenceDescriptor descriptor) {
        super("SharedReference XML Configurator"); 
        this.descriptor = descriptor;
        
        model = new ReferenceModel(descriptor);
        
        table = new JTable();
        generate = new JButton("Generate XML");
        close = new JButton("Close");
        name = new JTextField();
				fileName = new JTextField();
        
        name.setSize(new Dimension(200, 25));
				fileName.setSize(new Dimension(200, 25));
        
        table.setModel(model);
        
        table.getColumnModel().getColumn(1).setMinWidth(50);
        table.getColumnModel().getColumn(1).setMaxWidth(50);
        
        generate.addActionListener(new  ActionListener() {
            public void actionPerformed(ActionEvent e) {
                generate();
            };
        });
        
        close.addActionListener(new  ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close();
            };
        });
        
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(new JScrollPane(table), BorderLayout.CENTER);
        
        JPanel buttonsPanel = new JPanel();
        buttonsPanel.add(generate);
        buttonsPanel.add(close);
        
				JPanel topPanel = new JPanel(new BorderLayout());
				topPanel.setBorder(BorderFactory.createEtchedBorder());
				
				Box vBox = Box.createVerticalBox();
				topPanel.add(vBox, BorderLayout.CENTER);

				vBox.add(Box.createVerticalGlue());
				//vBox.add(Box.createVerticalStrut(10));

        Box box = Box.createHorizontalBox();
        box.add(new JLabel("Reference Name:"));
        box.add(Box.createHorizontalStrut(3));
        box.add(name);
        box.add(Box.createHorizontalGlue());
				
				vBox.add(box);
				vBox.add(Box.createVerticalStrut(10));
				
				box = Box.createHorizontalBox();
				box.add(new JLabel("Output fileName:"));
				box.add(Box.createHorizontalStrut(3));
				box.add(fileName);
        box.add(Box.createHorizontalGlue());
				vBox.add(box);
				//vBox.add(Box.createVerticalStrut(10));
				vBox.add(Box.createVerticalGlue());
        
        getContentPane().add(buttonsPanel, BorderLayout.SOUTH);
        getContentPane().add(topPanel, BorderLayout.NORTH);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            };
        });
    }
    
    public void generate() {
			try {
				FileOutputStream out = new FileOutputStream(fileName.getText(), false);
        descriptor.setName(name.getText());
        descriptor.writeXML(new PrintStream(out));
				JOptionPane.showMessageDialog(this, "XML file \""+ fileName.getText() + "\"generated successfully");
			} catch (FileNotFoundException e) {
				System.out.println("File \"" + fileName.getText() + "\"generated successfully");
				JOptionPane.showMessageDialog(this, e.getLocalizedMessage());
			}
    }
    
    public void close() {
        setVisible(false);
        dispose();
        System.exit(0);
    }
    
		public static final void main(String args[]) {
			InputStream xmlFile;
			FileInputStream classFile;
			
			
			try {
				classFile = loadClassFile(args);
				ShmemClassLoader loader = new ShmemClassLoader(classFile);
				ReferenceDescriptor descriptor = new ReferenceDescriptor(loader.getLoadedClass());
				Configurator configurator = new Configurator(descriptor);
				configurator.pack();
				configurator.setVisible(true);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
       
	public static FileInputStream loadClassFile(String args[]) 
	throws FileNotFoundException
	{
		int idx = 0; // start from 0
		while (idx < args.length) {
			if (args[idx].endsWith(".class")) {
				return new FileInputStream(args[idx]);
			}
			idx++;
		}
		return null;
	}
	
	public static InputStream loadXMLFile(String args[]) 
	throws FileNotFoundException
	{
		int idx = 1; // start from 1
		while (idx < args.length) {
			if (args[idx].endsWith(".xml")) {
				return new FileInputStream(args[idx]);
			}
			else if (args[idx].equals("-")) {
				return System.in;
			}
			idx++;
		}
		return null;
	}
}
