package br.shmem.compiler;

import java.lang.reflect.*;
import org.jdom.*;
import java.util.*;

public class ReferenceDescriptor {
    
    private List entries = new ArrayList();
    String name;
    
    public ReferenceDescriptor(Class c) 
			throws NotSerializableException
		{
			if (!java.io.Serializable.class.isAssignableFrom(c))
			throw new NotSerializableException(c);

        initEntries(c);
    }
		
    public void initEntries(Class c) {
        Method methods[] = c.getDeclaredMethods();
        
        for(int i = 0; i < methods.length; i++) {
            Method method = methods[i];
            
            int modifiers = method.getModifiers();
            
            if(Modifier.isPrivate(modifiers)) {
                continue;
            }
            
            if(Modifier.isProtected(modifiers)) {
                continue;
            }
            
            if(Modifier.isStatic(modifiers)) {
                continue;
            }
            
            if(Modifier.isAbstract(modifiers)) {
                continue;
            }
            
            if(Modifier.isNative(modifiers)) {
                continue;
            }
            
            entries.add(new Entry(method.toString()));
        }
    }
    
    public int getMethodsCount() {
        return entries.size();
    }
    
    public String getName() {
        return name;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public String getName(int index) {
        return ((Entry)entries.get(index)).getName();
    }
    
    public boolean isWrite(int index) {
        return ((Entry)entries.get(index)).isWrite();
    }
    
    public void setName(int index, String name) {
        ((Entry)entries.get(index)).setName(name);
    }
    
    public void setWrite(int index, boolean write) {
        ((Entry)entries.get(index)).setWrite(write);
    }
    
    public void writeXML(java.io.PrintStream out) {
        out.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
        out.println("<!DOCTYPE reference PUBLIC \"shmemreference\" \"http://www.linux.ime.usp.br/~tminc/shmem.dtd\">");
        out.println();
        out.println("<reference name=\"" + getName() + "\">");
        for(int i = 0; i < getMethodsCount(); i++)
            out.println("  <method name=\"" + getName(i) + "\" write = \"" + isWrite(i) + "\" />");
        out.println("</reference>");
    }
    
    static final class Entry {
        String name;
        boolean write;
        
        public Entry(String name) {
            this.name = name;
            this.write = false;
        }
        
        public void setName(String name) {
            this.name = name;
        }
        public void setWrite(boolean write) {
            this.write = write;
        }
        
        public String getName() {
            return name;
        }
        public boolean isWrite() {
            return write;
        }
    }
}
