package br.shmem.compiler;

import java.util.Hashtable;
import java.io.*;

public class ShmemClassLoader extends ClassLoader {
	private Hashtable classes = new Hashtable();
	
	private String loadedClassName = null;
	private Class loadedClass = null;
	
	public ShmemClassLoader(FileInputStream fi) {
		
		Class result = getClassFromFileInputStream(fi);
		if (result == null) {
			throw new ClassFormatError();
		}
		loadedClassName = result.getName();
		loadedClass = result;
		resolveClass(result);
		classes.put(loadedClassName, result);
	}
	
	public Class getLoadedClass() {
		return loadedClass;
	}
	
	private Class getClassFromFileInputStream(FileInputStream fi) {
		byte result[];
		try {
			result = new byte[fi.available()];
			fi.read(result);
			return defineClass(null, result, 0, result.length);
		} catch (Exception e) {
			
			/*
			* If we caught an exception, either the class wasnt found or it
			* was unreadable by our process.
			*/
			
			return null;
		}	
	}
	
	/**
	* This sample function for reading class implementations reads
	* them from the local file system
	*/
	private byte getClassImplFromFileSystem(String className)[] {
		//System.out.println("        >>>>>> Fetching the implementation of "+className);
		byte result[];
		try {
			FileInputStream fi = new FileInputStream(className);
			result = new byte[fi.available()];
			fi.read(result);
			return result;
		} catch (Exception e) {
			
			/*
			* If we caught an exception, either the class wasnt found or it
			* was unreadable by our process.
			*/
			
			return null;
		}
	}
	
	/**
	* This is a simple version for external clients since they
	* will always want the class resolved before it is returned
	* to them.
	*/
	public Class loadClass(String className) throws ClassNotFoundException {
		return (loadClass(className, true));
	}
	
	/**
	* This is the required version of loadClass which is called
	* both from loadClass above and from the internal function
	* FindClassFromClass.
	*/
	public synchronized Class loadClass(String className, boolean resolveIt)
	throws ClassNotFoundException {
		Class result;
		byte  classData[];
		
		//System.out.println("        >>>>>> Load class : "+className);
		
		/* Check our local cache of classes */
		result = (Class)classes.get(className);
		if (result != null) {
			//System.out.println("        >>>>>> returning cached result.");
			return result;
		}
		
		/* Check with the primordial class loader */
		try {
			result = super.findSystemClass(className);
			//System.out.println("        >>>>>> returning system class (in CLASSPATH).");
			return result;
		} catch (ClassNotFoundException e) {
			//System.out.println("        >>>>>> Not a system class.");
		}
		
		throw new ClassNotFoundException();
	}
}



