package wumpus;

import wumpus.agents.*;
import wumpus.gui.*;

import java.util.Random;

public class Board {
	
	public static final int NORTH = 0;
	public static final int EAST = 1;
	public static final int SOUTH = 2;
	public static final int WEST = 3;

	private Square[][] matrix;
	private boolean hasExit = false;
	private int monsterCount = 0;
	private int holeCount = 0;
	private Game game;
	private WumpusAgent wumpus;
	private BoardView view;
	
	public Board(int width, int height) {
		matrix = new Square[width][];
		for (int i = 0; i < width; i++) {
			matrix[i] = new Square[height];
		}
	}
	
	public void setView(BoardView view) {
		this.view = view;
	}
	
	public void fireRepaintRequest() {
		view.repaint();
	}
	
	public void setGame(Game game) {
		this.game = game;
	}
	
	public WumpusAgent getWumpusAgent() {
		return wumpus;
	}
	
	public void setWumpusAgent(WumpusAgent wumpus) {
		this.wumpus = wumpus;
	}
	
	public Game getGame() {
		return game;
	}
	
	public int getWidth() {
		return matrix.length;
	}
	
	public int getHeight() {
		return matrix[0].length;
	}
	
	public void workArrow(int x, int y) {
		wumpus.receiveArrow(x, y);
	}
	
	public Square getSquare(int x, int y) {
		Square result = null;
		try {
			result = matrix[x][y];
		} catch (ArrayIndexOutOfBoundsException e) {
			result = Square.WallSquare;
		}
		return result;
	}
	
	public boolean lay(int posX, int posY, Square square) {
		Square target = matrix[posX][posY];
		boolean allowed = false;
		if (target == null) {
			allowed = true;
		}
		else if (target.isHole()) {
			holeCount--;
			allowed = true;
		}
		else if (!target.isExit() && !target.isOccupied()) {
			allowed = true;
		}
		if (allowed) {
			if (square.isExit())
				hasExit = true;
			else if (square.isHole())
				holeCount++;
			matrix[posX][posY] = square;
		}
		return allowed;
	}
	
	public static void layRandomBoard(Board board) {
		int w = board.getWidth();
		int h = board.getHeight();
		Random rd = new Random();
		// lay Exit:
		int rw = rd.nextInt(w);
		int rh = rd.nextInt(h);
		board.lay(rw, rh, Square.ExitSquare);
		for (int i = 0; i < w; i++) {
			for (int j = 0; j < h; j++) {
				int goldOrHole = rd.nextInt(9);
				int goldAmount = 0;
				if (goldOrHole == 0) {// hole
					board.lay(i, j, Square.HoleSquare);
				}
				else {
					if (goldOrHole == 1) {// gold
						goldAmount = rd.nextInt(20) + 1;
						System.out.println("gold at (" + i + ", " + j + ")");
					}
					board.lay(i, j, new Square(goldAmount));
				}
			}
		}
	}
}
