package wumpus;

public class Square implements java.io.Serializable {

	public static final int NORMAL_TYPE = 0;
	public static final int HOLE_TYPE = 1;
	public static final int EXIT_TYPE = 2;
	public static final int WALL_TYPE = 3;
	
	private int squareType;
	private int goldAmount;
	private Object occupant;
	
	public static final Square ExitSquare = new Square(EXIT_TYPE, 0);
	public static final Square WallSquare = new Square(WALL_TYPE, 0);
	public static final Square HoleSquare = new Square(HOLE_TYPE, 0);
	
	protected Square(int squareType, int goldAmount) {
		this.squareType = squareType;
		this.goldAmount = goldAmount;
		this.occupant = null;
	}
	
	public Square(int goldAmount) {
		this(NORMAL_TYPE, goldAmount);
	}
	
	public int grabGold() {
		int tmp = goldAmount;
		goldAmount = 0;
		return tmp;
	}
	
	public int getGold() {
		return goldAmount;
	}
	
	public void putGold(int amount) {
		goldAmount = amount;
	}
	
	public boolean isOccupied() {
		return occupant != null;
	}
	
	public boolean isExit() {
		return squareType == EXIT_TYPE;
	}
	
	public boolean isHole() {
		return squareType == HOLE_TYPE;
	}
	
	public boolean isWall() {
		return squareType == WALL_TYPE;
	}
	
	public void occupy(Object someone) 
		throws OccupationException  
	{
		if (isWall()) {
			throw new OccupationException(WALL_TYPE);
		}
		else if (isHole()) {
			//throw new OccupationException(HOLE_TYPE);
		}
		else if (occupant != null) {
			throw new OccupationException(occupant);
		}
		occupant = someone;
	}
	
	public void leave(Object someone) {
		if (occupant.equals(someone)) {
			occupant = null;
		}
	}
	
	public Object getOccupant() {
		return occupant;
	}
}
