package wumpus;

import javax.swing.*;

public class Walker implements Serializable {
	protected Board board;
	private int posX;
	private int posY;
	private transient JComponent renderer;
	
	public Walker(Board board, int x, int y) {
		posX = x;
		posY = y;
		this.board = board;
	}
	
	protected void move(int toX, int toY) 
		throws OccupationException
	{
		Square sq = board.getSquare(toX, toY);
		Square old = board.getSquare(posX, posY);
		System.out.println("Moving from (" + posX + ", " + posY +
				") to (" + toX + ", " + toY + ")...");
		synchronized(sq) {
			sq.occupy(this);
			old.leave(this);
			posX = toX;
			posY = toY;
		}
		System.out.println("moving done.");
	}
	
	public void moveNorth() 
		throws OccupationException
	{
		move(posX, posY-1);
	}
	
	public void moveSouth() 
		throws OccupationException
	{
		move(posX, posY+1);
	}
	
	public void moveEast() 
		throws OccupationException
	{
		move(posX+1, posY);
	}
	
	public void moveWest() 
		throws OccupationException
	{
		move(posX-1, posY);
	}

	public void move(int direction) 
		throws OccupationException
	{
		if (direction == Board.EAST)
			moveEast();
		else if (direction == Board.WEST)
			moveWest();
		else if (direction == Board.SOUTH)
			moveSouth();
		else
			moveNorth();
	}
	
	public int getX() {
		return posX;
	}
	
	public int getY() {
		return posY;
	}
	
	public JComponent getRenderer() {
		return renderer;
	}
	
	public void setRenderer(JComponent r) {
		renderer = r;
	}
}
