package wumpus.agents;

import wumpus.*;
import java.util.Random;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class PlayerAgent implements  ActionListener {

	private Walker delegate = null;
	private Board board;
	private int gold = 0;
	private int arrows = 2;
	private int currentDirection = Board.NORTH;
	
	public PlayerAgent(Board board) {
		int x, y;
		this.board = board;
		Square s;
		Random lazyRandomizer = new Random();
		do {
			x = lazyRandomizer.nextInt(board.getWidth());
			y = lazyRandomizer.nextInt(board.getHeight());
			s = board.getSquare(x, y);
		} while (!s.isHole() && !s.isExit() && !s.isOccupied());
		delegate = new Walker(board, x, y);
		delegate.setRenderer(new PlayerRenderer(this));
		try {
			s.occupy(delegate);
		} catch (OccupationException e) {
			System.err.println("this should never happen!");
			e.printStackTrace(System.err);
		}
	}
	
	private int nextX() {
		if (currentDirection == Board.EAST)
			return delegate.getX() +1;
		else if (currentDirection == Board.WEST)
			return delegate.getX() - 1;
		else
			return delegate.getX();
	}
	
	private int nextY() {
		if (currentDirection == Board.SOUTH)
			return delegate.getY() +1;
		else if (currentDirection == Board.NORTH)
			return delegate.getY() - 1;
		else
			return delegate.getY();
	}
	
	protected void grabGold() {
		gold += board.getSquare(delegate.getX(), delegate.getY()).grabGold();		
	}
	
	protected void fireArrow() {
		if (arrows > 0) {
			arrows--;
			board.workArrow(nextX(), nextY());
		}
	}
	
	protected void climb() {
		board.getGame().setVictory();
	}
	
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("walk")) {
			try {
				delegate.move(currentDirection);
			} catch (OccupationException e) {
			}
		}
		else if (event.getActionCommand().equals("left")) {
			currentDirection = (currentDirection + 1) % 4;
		}
		else if (event.getActionCommand().equals("right")) {
			currentDirection = (currentDirection - 1) % 4;
		}
		else if (event.getActionCommand().equals("grab")) {
			grabGold();
		}
		else if (event.getActionCommand().equals("fire")) {
			fireArrow();
		}
		else if (event.getActionCommand().equals("climb")) {
			climb();
		}
		System.err.println("Player: position: " + delegate.getX() + ", " + delegate.getY());
		System.err.println("Player: direction: " + currentDirection);
		System.err.println("Player: arrows: " + arrows);
		board.fireRepaintRequest();
	}
	
	public Walker getWalkerDelegate() {
		return delegate;
	}
	
	class PlayerRenderer extends JComponent {
		
		protected PlayerAgent agent;
		
		public PlayerRenderer(PlayerAgent agent) {
			this.agent = agent;
		}
		
		public void paintComponent(Graphics g) {
			int w = getWidth();
			int h = getHeight();
			g.setColor(Color.blue);
			g.fillRect(w/4, h/4, w/2, h/2);
		}
	}
	
}
