package wumpus.gui;

import wumpus.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *  Description of the Class
 *
 *@author     tminc
 *@version
 */
public class BoardView extends JComponent implements ActionListener {

	/**
	 *  Description of the Field
	 *
	 *@since
	 */
	public final static int SQUARE_SIDE = 20;
	public final static Color HOLE_COLOR = new Color(0x008000);
	public final static Color WALL_COLOR = new Color(0x808000);
	
	private int bh;
	private Board board;
	private int bw;


	/**
	 *  Constructor for the BoardView object
	 *
	 *@param  board  Description of Parameter
	 *@since
	 */
	public BoardView(Board board) {
		this.board = board;
		board.setView(this);
		bh = board.getHeight();
		bw = board.getWidth();
		Dimension d = new Dimension((SQUARE_SIDE + 2) * (bw + 2), (SQUARE_SIDE + 2) * (bh + 2));
		setMinimumSize(d);
		setPreferredSize(d);
		setSize(d);
	}
	// pixels


	/**
	 *  Description of the Method
	 *
	 *@param  g  Description of Parameter
	 *@since
	 */
	public void paintComponent(Graphics g) {
		int x0 = 1;
		int y0 = 1;
		g.setColor(Color.black);
		g.fillRect(0, 0, getWidth(), getHeight());
		g.setColor(Color.white);
		for (int j = -1; j <= bh; j++) {
			for (int i = -1; i <= bw; i++) {
//				g.fillRect(x0, y0, SQUARE_SIDE, SQUARE_SIDE);
				paintSquare(i, j, x0, y0, g);
				x0 += 1 + SQUARE_SIDE;
			}
			x0 = 1;
			y0 += 1 + SQUARE_SIDE;
		}
	}


	/**
	 *  Description of the Method
	 *
	 *@param  i   Description of Parameter
	 *@param  j   Description of Parameter
	 *@param  x0  Description of Parameter
	 *@param  y0  Description of Parameter
	 *@param  g   Description of Parameter
	 *@since
	 */
	protected void paintSquare(int i, int j, int x0, int y0, Graphics g) {
		Square sq = board.getSquare(i, j);
		if (sq.isWall()) {
			g.setColor(WALL_COLOR);
			g.fillRect(x0, y0, SQUARE_SIDE, SQUARE_SIDE);
		} 
		else if (sq.isExit()) {
			g.setColor(Color.cyan);
			g.fillRect(x0, y0, SQUARE_SIDE, SQUARE_SIDE);			
		}
		else {
			g.setColor(Color.white);
			g.fillRect(x0, y0, SQUARE_SIDE, SQUARE_SIDE);

			if (sq.isHole()) {
				g.setColor(HOLE_COLOR);
				g.fillOval(x0 + SQUARE_SIDE/5, y0 + SQUARE_SIDE/3, 3*SQUARE_SIDE/5, SQUARE_SIDE/3);
			} else {
				if (sq.getGold() > 0) {
					g.setColor(Color.yellow);
					g.fillOval(x0 + SQUARE_SIDE/4, y0 + 3*SQUARE_SIDE/4, SQUARE_SIDE/4, SQUARE_SIDE/4);
				}
				if (sq.isOccupied()) {
					//System.out.println("Agent: (" + i + ", " + j + ")");
					Walker oc = (Walker) sq.getOccupant();
					JComponent comp = oc.getRenderer();
					if (comp == null) {
						g.setColor(Color.red);
						g.drawLine(x0, y0, x0 + SQUARE_SIDE, y0 + SQUARE_SIDE);
						g.drawLine(x0, y0 + SQUARE_SIDE, x0 + SQUARE_SIDE, y0);
					}
					else {
						comp.setBounds(x0, y0, SQUARE_SIDE, SQUARE_SIDE);
						comp.paint(g.create(x0, y0, SQUARE_SIDE, SQUARE_SIDE));
					}
				}
			}
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		repaint();
	}
}

