package wumpus.gui;

import wumpus.*;
import wumpus.agents.*;

import java.awt.*;
import javax.swing.*;

public class Main extends JFrame {
	
	private Game game;
	private WumpusAgent wumpus;
	private PlayerAgent player;
	
	private PlayerPanel playerPanel;
	private BoardView view;
	
	public Main() {
		super("Wumpus' world");
		setSize(600, 400);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		makeComponents();
		populate();
	}
	
	protected void makeComponents() {
		Container c = getContentPane();
		c.setLayout(new BorderLayout());
		Board board = new Board(10, 10);
		game = new Game(board);
		Board.layRandomBoard(board);
		view = new BoardView(board); 
		JScrollPane scrollpane = new JScrollPane(view);
		
		wumpus = new WumpusAgent(board);
		player = new PlayerAgent(board);
		playerPanel = new PlayerPanel(player);
		
		JSplitPane splitter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, view, playerPanel);
		
		c.add(splitter, BorderLayout.CENTER);
	}
	
	protected void populate() {
		Thread t = new Thread(wumpus);
		t.setDaemon(true);
		t.start();
		
		game.startGame();
	}
	
	public static final void main(String args[]) {
		Main m = new Main();
		m.setVisible(true);
		m.populate();
	}
}
