package wumpus.gui;


import wumpus.*;
import wumpus.agents.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class PlayerPanel extends JPanel {

	private JButton b_walk;
	private JButton b_left;
	private JButton b_right;
	private JButton b_fire;
	private JButton b_climb;
	private JButton b_grab;
	
	private JLabel currentGold;
	private JLabel smellsLight;
	private JLabel windyLight;
	private JLabel exitLight;
	
	protected PlayerAgent agent;
	
	public PlayerPanel(PlayerAgent agent) {
		this.agent = agent;
		makeComponents();
	}
	protected void makeComponents() {
		setLayout(new BorderLayout());
		Box b = Box.createVerticalBox();
		b.add(makeLights());
		b.add(makeButtons());
		add(b);
	}
	
	protected JLabel makeLightLabel(Color c) {
		JLabel l = new JLabel("x");
		l.setBackground(c);
		l.setBorder(BorderFactory.createLineBorder(Color.black));
		l.setMinimumSize(new Dimension(10, 10));
		l.setSize(10, 10);
		return l;
	}
	
	protected JPanel makeLights() {
		smellsLight = makeLightLabel(Color.green);
		windyLight = makeLightLabel(Color.blue);
		exitLight = makeLightLabel(Color.white);
		
		Box box = Box.createVerticalBox();
		Box hbox = Box.createHorizontalBox();

		box.add(Box.createVerticalGlue());
		hbox.add(smellsLight);
		hbox.add(Box.createHorizontalStrut(10));
		hbox.add(new JLabel("Smells funny"));

		box.add(hbox);
		box.add(Box.createVerticalStrut(10));
		
		hbox = Box.createHorizontalBox();
		hbox.add(windyLight);
		hbox.add(Box.createHorizontalStrut(10));
		hbox.add(new JLabel("Windy"));

		box.add(hbox);
		box.add(Box.createVerticalStrut(10));
		
		hbox = Box.createHorizontalBox();
		hbox.add(exitLight);
		hbox.add(Box.createHorizontalStrut(10));
		hbox.add(new JLabel("Light above your head"));

		box.add(Box.createVerticalGlue());
		JPanel p = new JPanel(new BorderLayout());
		p.setBorder(BorderFactory.createTitledBorder("Environment"));
		p.add(box, BorderLayout.CENTER);
		return p;
	}
	
	protected JPanel makeButtons() {
		b_walk = new JButton("walk");
		b_walk.setActionCommand("walk");
		b_walk.addActionListener(agent);

		b_left = new JButton("left");
		b_left.setActionCommand("left");
		b_left.addActionListener(agent);

		b_right = new JButton("right");
		b_right.setActionCommand("right");
		b_right.addActionListener(agent);

		b_fire = new JButton("fire");
		b_fire.setActionCommand("fire");
		b_fire.addActionListener(agent);

		b_climb = new JButton("climb");
		b_climb.setActionCommand("climb");
		b_climb.addActionListener(agent);

		b_grab = new JButton("grab");
		b_grab.setActionCommand("grab");
		b_grab.addActionListener(agent);
		
		JPanel p = new JPanel(new GridLayout(0, 1, 5, 10));
		p.add(b_walk);
		JPanel p1 = new JPanel(new GridLayout(1, 0, 5, 0));
		p1.add(b_left);
		p1.add(b_right);
		p.add(p1);
		p.add(b_fire);
		p.add(b_grab);
		
		return p;
	}
	
}
